{% extends '!footer.html' %}
{% block extrafooter %}
<p>Covasim is licensed under the <a href="https://creativecommons.org/licenses/by-nc-sa/4.0/legalcode">Creative Commons
    Attribution-Noncommercial-ShareAlike 4.0 License</a>.</p>
<p>Send documentation feedback to <a href="mailto:feedback@idmod.org" target="_top">feedback@idmod.org</a>. If you have
    questions, email <a href="mailto:support@idmod.org" target="_top">support@idmod.org</a>.</p>
<p><a href="https://www.gatesfoundation.org/Privacy-and-Cookies-Notice">Privacy and Cookies Notice</a>&nbsp;&nbsp;&nbsp;|&nbsp;&nbsp;&nbsp;
<a href="https://www.gatesfoundation.org/Terms-of-Use">Terms of Use</a></p>
{% endblock %}
