{% extends "!page.html" %}

{% set css_files = css_files + ["_static/theme_overrides.css"] %}

{% block htmltitle %}
{{ super() }}

{% if not READTHEDOCS %}
<meta name="robots" content="none">
{% endif %}

{% endblock %}

{% block extrahead %}

<link href="https://fonts.googleapis.com/css?family=Lato|Montserrat|Open+Sans" rel="stylesheet">

{% endblock %}
