# -*- coding: utf-8 -*-

__all__ = ['__version__', '__versiondate__', '__license__']

__version__ = '1.7.4'
__versiondate__ = '2020-10-02'
__license__ = f'Covasim {__version__} ({__versiondate__}) — © 2020 by IDM'
